<?php
namespace FiloBlu\Refilo\Model;
use FiloBlu\Refilo\Api\CustomerGroupManagmentInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Model\Customer;
use Psr\Log\LoggerInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Customer\Model\CustomerRegistry;
use Magento\Customer\Model\CustomerFactory as CustomerModelFactory;
use Magento\Quote\Api\Data\CartInterfaceFactory;
use Magento\Customer\Model\Session;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\App\Http\Context;

class CustomerGroupManagment implements CustomerGroupManagmentInterface
{
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;
    /**
     * @var CustomerRegistry
     */
    private $customerRegistry;
    /**
     * @var CustomerModelFactory
     */
    private $customerModelFactory;
    /**
     * @var CartInterfaceFactory
     */
    private $cartInterfaceFactory;
    /**
     * @var Session
     */
    private $customerSession;
    /**
     * @var CheckoutSession
     */
    private $checkoutSession;
    /**
     * @var Context
     */
    private $httpContext;

    /**
     * CustomerGroupManagment constructor.
     * @param CustomerRepositoryInterface $customerRepository
     * @param LoggerInterface $logger
     * @param CartRepositoryInterface $cartRepository
     * @param CustomerRegistry $customerRegistry
     * @param CustomerModelFactory $customerModelFactory
     * @param CartInterfaceFactory $cartInterfaceFactory
     * @param Session $customerSession
     * @param CheckoutSession $checkoutSession
     * @param Context $httpContext
     */
    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        LoggerInterface $logger,
        CartRepositoryInterface  $cartRepository,
        CustomerRegistry $customerRegistry,
        CustomerModelFactory $customerModelFactory,
        CartInterfaceFactory  $cartInterfaceFactory,
        Session  $customerSession,
        CheckoutSession $checkoutSession,
        Context $httpContext){
        $this->customerRepository = $customerRepository;
        $this->logger = $logger;
        $this->cartRepository = $cartRepository;
        $this->customerRegistry = $customerRegistry;
        $this->customerModelFactory = $customerModelFactory;
        $this->cartInterfaceFactory = $cartInterfaceFactory;
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->httpContext = $httpContext;
    }

    /**
     * @param int $customerId
     * @param int $customerGroupId
     * @return CustomerInterface
     */
    public function updateCustomerGroup(int $customerId, int $customerGroupId)
    {
        try{
            try{
                $customer = $this->customerRepository->getById($customerId);
            }catch (\Exception $e){
                $customer = $this->pushCustomerIntoRegistry($customerId);
            }catch (\Throwable $t){
                $customer = $this->pushCustomerIntoRegistry($customerId);
            }

            if($customer && $customer->getGroupId() != $customerGroupId){
                $cart = $this->cartRepository->getActiveForCustomer($customerId,[$customer->getStoreId()]);
                $customer->setGroupId($customerGroupId);
                $this->customerRepository->save($customer);
                $cart->setCustomer($customer);
                $this->cartRepository->save($cart);
            }
            return $customer;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
        }
    }

    /**
     * @param int $customerId
     * @return CustomerInterface
     */
    protected function pushCustomerIntoRegistry(int $customerId){
        try{
            $customer = $this->customerModelFactory->create()->load($customerId);
            $this->customerRegistry->push($customer);
            $account = $this->customerRepository->getById($customerId);
            $this->customerSession->regenerateId();
            $newQuote =$this->cartInterfaceFactory->create()->setCustomer($account)->setIsActive(true)->setStoreId($customer->getStoreId())->save();
            $this->checkoutSession->setQuoteId($newQuote->getId());
            $this->customerSession->setCustomerId($customer->getId());
            $this->httpContext->setValue(\Magento\Customer\Model\Context::CONTEXT_AUTH, true, false);
            $this->customerSession->setCustomerData($account);
            $customerModel = $this->customerModelFactory->create()->updateData($account);
            $this->customerSession->setCustomer($customerModel);
            return $account;
        }catch (\Exception $e){
            $this->logger->critical("PushCustomerIntoRegistry failed: ".$e->getMessage());
        }catch (\Throwable $t){
            $this->logger->critical("PushCustomerIntoRegistry failed: ".$t->getMessage());
        }
    }
}
